@@ -29,6 +29,7 @@
 #include "gnome-vfs-cancellable-ops.h"
 #include "gnome-vfs-method.h"
 #include "gnome-vfs-private-utils.h"
+#include "gnome-vfs-utils.h"
 #include "gnome-vfs-handle-private.h"
 
 #include <glib/gmessages.h>
@@ -54,8 +55,8 @@ gnome_vfs_open_uri_cancellable (GnomeVFSHandle **handle,
 	if (!VFS_METHOD_HAS_FUNC(uri->method, open))
 		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 
-	result = uri->method->open (uri->method, &method_handle, uri, open_mode,
-				    context);
+	result = (uri->method->open) (uri->method, &method_handle, uri, open_mode,
+				      context);
 
 	if (result != GNOME_VFS_OK)
 		return result;
@@ -296,8 +297,13 @@ gnome_vfs_find_directory_cancellable (GnomeVFSURI *near_uri,
 	if (near_uri != NULL) {
 		gnome_vfs_uri_ref (near_uri);
 	} else {
+		char *text_uri;
+
+		text_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
+		g_assert (text_uri != NULL);
 		/* assume file: method and the home directory */
-		near_uri = gnome_vfs_uri_new (g_get_home_dir());
+		near_uri = gnome_vfs_uri_new (text_uri);
+		g_free (text_uri);
 	}
 
 	g_assert (near_uri != NULL);
